package com.eitc.web.controller.common;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.concurrent.TimeUnit;
import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.FastByteArrayOutputStream;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import com.google.code.kaptcha.Producer;
import com.eitc.common.config.EitcConfig;
import com.eitc.common.constant.CacheConstants;
import com.eitc.common.constant.Constants;
import com.eitc.common.core.domain.AjaxResult;
import com.eitc.common.core.redis.RedisCache;
import com.eitc.common.utils.sign.Base64;
import com.eitc.common.utils.uuid.IdUtils;
import com.eitc.system.service.ISysConfigService;

/**
 * 验证码操作处理
 * 
 * @author eitc
 */
@RestController
public class CaptchaController
{
    @Resource(name = "captchaProducer")
    private Producer captchaProducer;

    @Resource(name = "captchaProducerMath")
    private Producer captchaProducerMath;

    @Autowired
    private RedisCache redisCache;
    
    @Autowired
    private ISysConfigService configService;
    /**
     * 生成验证码
     */
    @GetMapping("/captchaImage")
    public AjaxResult getCode(HttpServletResponse response) throws IOException
    {
        AjaxResult ajax = AjaxResult.success();
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        ajax.put("captchaEnabled", captchaEnabled);
        if (!captchaEnabled)
        {
            return ajax;
        }
        int width = 200;
        int height = 50;
        // 保存验证码信息
        String uuid = IdUtils.simpleUUID();
        String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + uuid;

        String capStr = null, code = null;
        BufferedImage image = null;

        // 生成验证码
        String captchaType = EitcConfig.getCaptchaType();
        if ("math".equals(captchaType))
        {
            String capText = captchaProducerMath.createText();
            capStr = capText.substring(0, capText.lastIndexOf("@"));
            code = capText.substring(capText.lastIndexOf("@") + 1);
//            image = captchaProducerMath.createImage(capStr);
            image = new BufferedImage(150, 50, BufferedImage.TYPE_INT_RGB);
            Graphics2D g2d = image.createGraphics();
            // 设置背景颜色
            g2d.setColor(Color.WHITE);
            g2d.fillRect(0, 0, width, height);

            // 设置字体和颜色
            Font font = new Font("Arial", Font.BOLD, 20);
            g2d.setFont(font);
            g2d.setColor(Color.BLACK);
            g2d.drawString(capStr, 10, 30);

            // 释放图形上下文使用的系统资源
            g2d.dispose();
        }
        else if ("char".equals(captchaType))
        {
            capStr = code = captchaProducer.createText();
            image = new BufferedImage(150, 50, BufferedImage.TYPE_INT_RGB);
            Graphics2D g2d = image.createGraphics();
            // 设置背景颜色
            g2d.setColor(Color.WHITE);
            g2d.fillRect(0, 0, width, height);

            // 设置字体和颜色
            Font font = new Font("Arial", Font.BOLD, 20);
            g2d.setFont(font);
            g2d.setColor(Color.BLACK);
            g2d.drawString(capStr, 10, 30);

            // 释放图形上下文使用的系统资源
            g2d.dispose();
        }

        redisCache.setCacheObject(verifyKey, code, Constants.CAPTCHA_EXPIRATION, TimeUnit.MINUTES);
        // 转换流信息写出
        FastByteArrayOutputStream os = new FastByteArrayOutputStream();
        try
        {
            ImageIO.write(image, "jpg", os);
        }
        catch (IOException e)
        {
            return AjaxResult.error(e.getMessage());
        }

        ajax.put("uuid", uuid);
        ajax.put("img", Base64.encode(os.toByteArray()));
        return ajax;
    }
}
