package com.eitc.common.filter;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.eitc.common.config.ParameterRequestWrapper;
import com.eitc.common.constant.HttpConst;
import com.eitc.common.utils.CryptoUtil;
import com.eitc.common.utils.StringUtils;
import org.springframework.http.MediaType;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;










/**
 * Repeatable 过滤器
 *
 * @author eitc
 */
public class RepeatableFilter implements Filter {

    private static final String DECRYPT_PARAM_NAME = "decrypt";//请求参数包含的是否加密的字段

    private static final String DEFAULT_DECRYPT_TYPE = "AES";//默认加密的类型

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {

    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse res = (HttpServletResponse) response;
        {
            String contentType = req.getContentType();//获取contentType请求头
            if(StringUtils.isNotBlank(contentType)){
                String method = req.getMethod();//获取请求方法  post/get
                //2 处理post请求  只处理application/x-www-form-urlencoded  application/json,对于multipart/form-data，直接放行
                if (method.trim().equalsIgnoreCase(HttpConst.POST_METHOD) || method.trim().equalsIgnoreCase(HttpConst.PUT_METHOD)) {
                    if (contentType.trim().toLowerCase().contains(HttpConst.MULTIPART_CONTENT_TYPE)) {
                        chain.doFilter(req, response);
                        return;
                    }
                    //处理application/x-www-form-urlencoded
                    if (contentType.trim().toLowerCase().contains(HttpConst.FORM_URLENCODED_CONTENT_TYPE)) {
                        String decrypt = request.getParameter(DECRYPT_PARAM_NAME);
                        if(decrypt==null||decrypt.trim().isEmpty()){
                            chain.doFilter(request, response);
                            return;
                        }
                        chain.doFilter(req, response);
                        return;
                    }
                    //处理application/json
                    if (contentType.trim().toLowerCase().contains(HttpConst.JSON_CONTENT_TYPE)) {
                        ParameterRequestWrapper requestWrapper = new ParameterRequestWrapper(req);
    //                    String collect = requestWrapper.getReader().lines().collect(Collectors.joining(System.lineSeparator()));
    //                    // 解密
    //                    String decrypt = CryptoUtil.decrypt(collect);
    //                    System.out.println(decrypt);
    //                    byte[] bytes = decrypt.getBytes(StandardCharsets.UTF_8);
    //                    ByteArrayOutputStream binaryStream = new ByteArrayOutputStream();
    //                    try {
    //                        binaryStream.write(bytes);
    //                        byte[] byteArray = binaryStream.toByteArray();
    //                        requestWrapper.setInputStream(byteArray);
    //                    } catch (IOException e) {
    //                        e.printStackTrace();
    //                    } finally {
    //                        try {
    //                            binaryStream.close();
    //                        } catch (IOException e) {
    //                            e.printStackTrace();
    //                        }
    //                    }
                        chain.doFilter(requestWrapper, response);
                        return;
                    }
                }
            }
        }


        HttpServletRequest requestWrapper = null;
        if (req instanceof HttpServletRequest
                && StringUtils.startsWithIgnoreCase(req.getContentType(), MediaType.APPLICATION_JSON_VALUE)) {
            requestWrapper = new RepeatedlyRequestWrapper((HttpServletRequest) request, response);
        }
        if (null == requestWrapper) {
            chain.doFilter(req, res);
        } else {
            chain.doFilter(requestWrapper, res);
        }
    }


    /**
     * 字符串解密
     *
     * @param value
     * @return
     */
    public String decryptParam(String value, String decryptType) {
        if (decryptType.trim().equalsIgnoreCase(DEFAULT_DECRYPT_TYPE)) {
//            return AESCipher.decryptAES(value,AESCipher.KEY);
            return "111";
        } else {
            return value + "+++";
        }

    }

    /**
     * 普通的post/get请求
     *
     * @param parameterMap
     */
    public Map<String, String[]> decryptParamForNormalRequest(Map<String, String[]> parameterMap, String decryptType) {
        Map<String, String[]> decryptMap = new HashMap<>();
        if (parameterMap == null || parameterMap.size() == 0) {
            return decryptMap;
        }
        for (Map.Entry<String, String[]> entry : parameterMap.entrySet()) {
            String key = entry.getKey();
            if (key == null || key.trim().equalsIgnoreCase(DECRYPT_PARAM_NAME)) {
                continue;
            }
            String[] value = entry.getValue();
            //String decryptKey = decodeParam(key,decryptType);
            String[] decryptValue = null;
            if (value != null && value.length > 0) {
                decryptValue = new String[value.length];
                for (int i = 0; i < value.length; i++) {
                    decryptValue[i] = decryptParam(value[i], decryptType);
                }
            }
            decryptMap.put(key, decryptValue);
        }
        //打印用
        StringBuffer printStr = new StringBuffer();
        for (Map.Entry<String, String[]> entry1 : decryptMap.entrySet()) {
            printStr.append(entry1.getKey()).append("=").append(Arrays.asList(entry1.getValue())).append("&");
        }
        System.out.println("ParamsFilter:发送的请求参数:" + JSON.toJSONString(printStr));
        return decryptMap;
    }

    /**
     * post的application/json请求
     *
     * @param body
     */
    public String decryptParamForPostJsonRequest(String body, String decryptType) {
        String decryptBody = "{}";
        if (body == null || body.trim().isEmpty() || body.trim().equalsIgnoreCase("{}") || !body.trim().contains(":")) {
            return decryptBody;
        }
        Map<String, Object> map = JSON.parseObject(body, new TypeReference<Map<String, Object>>() {
        });
        if (map == null || map.size() == 0) {
            return decryptBody;
        }
        Map<String, Object> decryptMap = new HashMap<>();
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            String key = entry.getKey();
            if (key == null || key.trim().equalsIgnoreCase(DECRYPT_PARAM_NAME)) {
                continue;
            }
            Object value = entry.getValue();
            String valueStr = String.valueOf(value);
            if (valueStr == null || valueStr.trim().isEmpty() || valueStr.trim().equalsIgnoreCase("null")) {
                valueStr = null;
            }
            decryptMap.put(key, decryptParam(valueStr, decryptType));
        }
        decryptBody = JSON.toJSONString(decryptMap);
        System.out.println("ParamsFilter:发送的请求参数:" + decryptBody);
        return decryptBody;
    }

    @Override
    public void destroy() {

    }


}
