package com.eitc.patient.domain;

import java.util.Date;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.eitc.common.core.domain.EitcBaseEntity;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 患者回访对象 erm_patient_follow_up
 *
 * @author eitc
 * @date 2024-07-05
 */
@EqualsAndHashCode(callSuper = true)
@Data
@ApiModel("患者回访对象")
@TableName("erm_patient_follow_up")
public class PatientFollowUp extends EitcBaseEntity {

    /**
     * 回访人：默认当前账号姓名，可修改
     */
    @TableField(value = "follow_up_user_id")
    @ApiModelProperty(value = "回访人", name = "followUpUserId")
    private String followUpUserId;

    /**
     * $column.columnComment
     */
    @TableField(value = "follow_up_user")
    @ApiModelProperty(value = "回访人", name = "followUpUser")
    private String followUpUser;

    /**
     * 就诊表外键
     */
    @TableField(value = "patient_appointment_id")
    @ApiModelProperty(value = "就诊表外键", name = "patientAppointmentId")
    private String patientAppointmentId;

    /**
     * 就诊时间：获取患者最新就诊记录时间，如未就诊，展示空状态即可
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "appointment_start_time")
    @ApiModelProperty(value = "就诊时间", name = "appointmentStartTime")
    private Date appointmentStartTime;

    /**
     * 回访时间：默认当前日期及时间，可修改
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "follow_up_time")
    @ApiModelProperty(value = "回访时间", name = "followUpTime")
    private Date followUpTime;

    /**
     * 回访状态：已回访、待回访
     */
    @TableField(value = "follow_up_status")
    @ApiModelProperty(value = "回访状态", name = "followUpStatus")
    private Integer followUpStatus;

    /**
     * 回访类型：未成交、已上门、术后
     */
    @TableField(value = "follow_up_type")
    @ApiModelProperty(value = "回访类型", name = "followUpType")
    private Integer followUpType;

    /**
     * 回访内容：治疗一周回访、拔牙一天回访
     */
    @TableField(value = "follow_up_content")
    @ApiModelProperty(value = "回访内容", name = "followUpContent")
    private String followUpContent;

    /**
     * 回访结果：无不适、数量出血、牙微疼痛
     */
    @TableField(value = "follow_up_result")
    @ApiModelProperty(value = "回访结果", name = "followUpResult")
    private String followUpResult;

}
