package com.eitc.patient.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.eitc.patient.domain.MedicalRecord;
import com.eitc.patient.dto.MedicalRecordHistoryDto;
import com.eitc.patient.dto.MedicalRecordListDto;
import com.eitc.patient.param.MedicalRecordSaveParam;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * MedicalRecordService
 *
 * @author mtx
 * @date: 2024/05/17 15:37
 */
@Service
public interface IMedicalRecordService extends IService<MedicalRecord> {

    /**
     * 保存病历
     *
     * @param vo 参数
     * @author mtx
     * @date: 2024/5/22 10:00
     */
    void addOrEdit(MedicalRecordSaveParam vo);

    /**
     * 病历列表
     *
     * @param searchParam 病历信息
     * @return 病历列表
     * @author mtx
     * @date: 2024/5/23 9:17
     */
    List<MedicalRecordListDto> medicalRecordList(String searchParam);

    /**
     * 统计诊断分类
     *
     * @return 统计结果
     * @author mtx
     * @date: 2024/6/5 9:10
     */
    List<Map<String, String>> countByDiagnosisTypeName();

    /**
     * 根据牙齿位置检查访问类型
     *
     * @return 返回访问类型，具体类型根据业务逻辑确定
     */
    List<MedicalRecord> findListByToothPosition(MedicalRecordHistoryDto dto);

}
