package com.eitc.patient.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.eitc.patient.domain.Patient;
import com.eitc.patient.vo.PatientVO;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * PatientService
 *
 * @author mtx
 * @date: 2024/05/17 15:37
 */
@Service
public interface IPatientService extends IService<Patient> {

    /**
     * 添加患者
     *
     * @param vo 参数
     * @author mtx
     * @date: 2024/5/22 9:12
     */
    Patient addOrEdit(PatientVO vo);

    /**
     * 删除患者
     *
     * @param ids ids
     * @return 状态
     * @author mtx
     * @date: 2024/5/22 9:40
     */
    boolean delete(List<String> ids);

    /**
     * 获取一个新的用户编号，确保该用户编号在系统中是唯一的。
     *
     * 本方法通过调用PatientUtil的findMedicalRecordNum方法来获取一个初始的用户编号，
     * 然后检查系统中是否已存在该用户编号，如果存在，则继续调用findMedicalRecordNum获取新的用户编号，
     * 直到找到一个在系统中不存在的用户编号为止。
     * 这种方式可以保证返回的用户编号在系统中是唯一的，避免了用户编号重复的问题。
     *
     * @return 新的、在系统中唯一的用户编号。
     */
    String findMedicalRecordNum();

}
